﻿using Infragistics.Controls.Charts;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Security.Permissions;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace XamDataChartNoiseDataDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public PidDataSet pdRPM1K;
        public PidDataSet pdRPM10K;
        public PidDataSet pdRPM100K;
        public PidDataSet pdMAP1K;
        public PidDataSet pdMAP10K;
        public PidDataSet pdMAP100K;
        public PidDataSet pdTPS1K;
        public PidDataSet pdTPS10K;
        public PidDataSet pdTPS100K;
        public PidDataSet pdMAF1K;
        public PidDataSet pdMAF10K;
        public PidDataSet pdMAF100K;
        public PidDataSet crank1;
        public PidDataSet crank2;

        public Stopwatch sw = new Stopwatch();
        public long et = 0;
        public long et1 = 0;
        public MainWindow()
        {
            InitializeComponent();
            LoadDataSets();

            igChartPage.EngineDataChart.RefreshCompleted += EngineDataChart_RefreshCompleted;
            SizeChanged += MainWindow_SizeChanged;
        }

        private void MainWindow_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            //sw.Restart();
            et1 = sw.ElapsedMilliseconds;
            appendText("Size Changed", et1);
        }

        private void EngineDataChart_RefreshCompleted(object sender, EventArgs e)
        {
            et1 = sw.ElapsedMilliseconds;
            appendText("Refresh Completed", et1);
        }

        private void LoadDataSets()
        {
            pdRPM1K = new PidDataSet("EngineSpeed", ChartTestData.EngineSpeed, 0, 8000, 0);
            pdMAP1K = new PidDataSet("MAPSensor", ChartTestData.MAPSensor, 0, 105, 0);
            pdTPS1K = new PidDataSet("TPSPercent", ChartTestData.TPSPercent, 0, 100, 0);
            pdMAF1K = new PidDataSet("MassAirflow", ChartTestData.MassAirflow, 0, 600, 0);

            pdRPM10K = new PidDataSet("EngineSpeed", ChartTestData.EngineSpeed, 0, 8000, 10);
            pdMAP10K = new PidDataSet("MAPSensor", ChartTestData.MAPSensor, 0, 105, 10);
            pdTPS10K = new PidDataSet("TPSPercent", ChartTestData.TPSPercent, 0, 100, 10);
            pdMAF10K = new PidDataSet("MassAirflow", ChartTestData.MassAirflow, 0, 600, 10);

            pdRPM100K = new PidDataSet("EngineSpeed", ChartTestData.EngineSpeed, 0, 8000, 100);
            pdMAP100K = new PidDataSet("MAPSensor", ChartTestData.MAPSensor, 0, 105, 100);
            pdTPS100K = new PidDataSet("TPSPercent", ChartTestData.TPSPercent, 0, 100, 100);
            pdMAF100K = new PidDataSet("MassAirflow", ChartTestData.MassAirflow, 0, 600, 100);

            crank1 = new PidDataSet("Crank1", ChartTestData.GenerateCrankshaftSignal2(), -1.1, 1.1, 0);
            crank2 = new PidDataSet("Crank2", ChartTestData.GenerateCrankshaftSignal3(), -1.1, 1.1, 0);



        }

        //
        // Add a new plot of PID data to the chart
        // the pid name is used to track the plot instance
        //
        private void AddEngDataPlot(PidDataSet pid)
        {
            string seriesName = pid.Name;

            //assign a new axis else it will scale to existing axis
            //Assign min max to the value form the units selection
            var yAxis = new NumericYAxis();
            yAxis.Name = seriesName;
            yAxis.MaximumValue = pid.MaxDispVal;
            yAxis.MinimumValue = pid.MinDispVal;

            //show the Y lables for the FIRST ITME ONLY else use defaults
            //count is initially two because XAML definition adds them.  May not be needed?
            if (igChartPage.EngineDataChart.Axes.Count > 2)
            {
                //LabelSettings must be instantiated before we can access any properties
                yAxis.Visibility = Visibility.Collapsed;  //turn off the gridlines
                yAxis.LabelSettings = new AxisLabelSettings();
                yAxis.LabelSettings.Visibility = Visibility.Collapsed;
            }

            //Add X-axis "stuff"
            //igChartPage.commonXAxis.DataContext = pid;
            //igChartPage.commonXAxis.UpdateRange();

            //assign the series.  Give it a unique name so we can prevent duplication and remove when deselected.
            var series = new ImmediateLineSeries.ImmediateLineSeries();
            series.Name = seriesName;
            series.ItemsSource = pid.PlotData2;
            series.XAxis = igChartPage.commonXAxis;  //defined in XAML
            series.YAxis = yAxis;
            series.Thickness = 2;
            series.Resolution = 0.001;
            series.UseHairline = true;
            series.UpscaleOPD = true;
            series.XMemberPath = "X";
            series.YMemberPath = "Y";
            series.Brush = GetPlotBrush(pid.Name);

            //series.MarkerType = MarkerType.None;

            //Add this new series an Y axis
            igChartPage.EngineDataChart.Axes.Add(yAxis);
            igChartPage.EngineDataChart.Series.Add(series);

            // sciChartPage.lineRenderSeries.DataSeries = pid.DataSeries;
            //sciChartPage.sciChart.ZoomExtents();
        }


        private Brush GetPlotBrush(string name)
        {
            SolidColorBrush br = new SolidColorBrush();
            switch (name)
            {
                case "EngineSpeed":
                    br.Color = Colors.Blue;
                    break;
                case "MAPSensor":
                    br.Color = Colors.Violet;
                    break;
                case "TPSPercent":
                    br.Color = Colors.Cyan;
                    break;
                case "MassAirflow":
                    br.Color = Colors.Red;
                    break;
                default:
                    br.Color = Colors.Brown;
                    break;
            }
            return br;

        }


        //
        //Remove a data plot form the chart
        // Uses the series name to locate the item and remove it form the collection
        //
        private void RemoveEngDataPlot(string name)
        {
            if (igChartPage == null || igChartPage.EngineDataChart == null) return;

            //remove first line series with this name
            string seriesName = name;
            foreach (ImmediateLineSeries.ImmediateLineSeries var in igChartPage.EngineDataChart.Series)
            {
                if (var.Name == seriesName)
                {
                    igChartPage.EngineDataChart.Series.Remove(var);
                    break;
                }
            }

            //remove first axis with this name
            foreach (Axis var in igChartPage.EngineDataChart.Axes)
            {
                if (var.Name == seriesName)
                {
                    igChartPage.EngineDataChart.Axes.Remove(var);
                    break;
                }
            }

        }
        //
        // change the chart data size and redraw all selected items
        // this is clumsy, but it is just a demo...
        //
        private void rb1Ksamples_Checked(object sender, RoutedEventArgs e)
        {
            RadioButton rb = (sender as RadioButton);

            //remove all existing plot items
            RemoveEngDataPlot("EngineSpeed");
            RemoveEngDataPlot("MAPSensor");
            RemoveEngDataPlot("TPSPercent");
            RemoveEngDataPlot("MassAirflow");
            RemoveEngDataPlot("Crank1");
            RemoveEngDataPlot("Crank2");

            //plot the selected items
            sw.Restart();
            if (cbLoadChart1.IsChecked == true)
            {
                if (rb.Name == "rb1Ksamples") { AddEngDataPlot(pdRPM1K); }
                else if (rb.Name == "rb10Ksamples") { AddEngDataPlot(pdRPM10K); }
                else if (rb.Name == "rb100Ksamples") { AddEngDataPlot(pdRPM100K); }
            }
            if (cbLoadChart2.IsChecked == true)
            {
                if (rb.Name == "rb1Ksamples") { AddEngDataPlot(pdMAP1K); }
                else if (rb.Name == "rb10Ksamples") { AddEngDataPlot(pdMAP10K); }
                else if (rb.Name == "rb100Ksamples") { AddEngDataPlot(pdMAP100K); }
            }
            if (cbLoadChart3.IsChecked == true)
            {
                if (rb.Name == "rb1Ksamples") { AddEngDataPlot(pdTPS1K); }
                else if (rb.Name == "rb10Ksamples") { AddEngDataPlot(pdTPS10K); }
                else if (rb.Name == "rb100Ksamples") { AddEngDataPlot(pdTPS100K); }
            }
            if (cbLoadChart4.IsChecked == true)
            {
                if (rb.Name == "rb1Ksamples") { AddEngDataPlot(pdMAF1K); }
                else if (rb.Name == "rb10Ksamples") { AddEngDataPlot(pdMAF10K); }
                else if (rb.Name == "rb100Ksamples") { AddEngDataPlot(pdMAF100K); }
            }

            if (cbLoadCrank1.IsChecked == true)
            {
                AddEngDataPlot(crank1);
            }

            if (cbLoadCrank2.IsChecked == true)
            {
                AddEngDataPlot(crank2);
            }


            et1 = sw.ElapsedMilliseconds;
            appendText("Selected plots added", et1);
        }


        #region Check Box Events
        //these are currently unused
        private void cbLoadChart1_Checked(object sender, RoutedEventArgs e)
        {

        }

        private void cbLoadChart2_Checked(object sender, RoutedEventArgs e)
        {

        }

        private void cbLoadChart3_Checked(object sender, RoutedEventArgs e)
        {

        }
        private void cbLoadChart4_Checked(object sender, RoutedEventArgs e)
        {

        }

        private void cbLoadCrank1_Checked(object sender, RoutedEventArgs e)
        {

        }

        private void cbLoadCrank2_Checked(object sender, RoutedEventArgs e)
        {

        }

        private void cbLoadCrank3_Checked(object sender, RoutedEventArgs e)
        {

        }
        #endregion

        private void cmdClear_Click(object sender, RoutedEventArgs e)
        {
            txtInfo.Clear();
            sw.Restart();
        }

        //txt control is null until initialization is complete?
        private void appendText(string msg, long timestamp)
        {
            if (txtInfo == null) return;
            txtInfo.AppendText(string.Format("{0:00000} mS:  {1} {2}", et1, msg, Environment.NewLine));
            txtInfo.ScrollToEnd();
            DoEvents();
        }

        private void cmdSetDefaultSize_Click(object sender, RoutedEventArgs e)
        {
            et1 = sw.ElapsedMilliseconds;
            appendText("Requested Default Size", et1);
            this.Width = 700;
            this.Height = 400;
        }

        private void cmdSetBigSize_Click(object sender, RoutedEventArgs e)
        {
            et1 = sw.ElapsedMilliseconds;
            appendText("Requested Big Size", et1);
            this.Width = 1400;
            this.Height = 800;
        }


        #region Do Events Dispatcher
        // Simulate a "DoEvents" for handling updates during non-background short operations
        // https://msdn.microsoft.com/en-us/library/system.windows.threading.dispatcherframe.aspx?f=255&MSPPError=-2147217396
        //
        [SecurityPermissionAttribute(SecurityAction.Demand, Flags = SecurityPermissionFlag.UnmanagedCode)]
        public void DoEvents()
        {
            //DispatcherFrame frame = new DispatcherFrame();
            //Dispatcher.CurrentDispatcher.BeginInvoke(DispatcherPriority.Background,
            //    new DispatcherOperationCallback(ExitFrame), frame);
            //Dispatcher.PushFrame(frame);
        }

        public object ExitFrame(object f)
        {
            ((DispatcherFrame)f).Continue = false;

            return null;
        }
        #endregion
    }
}
